#!/usr/bin/env node
"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var log_1 = require("./log");
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var distinguisher_1 = require("./distinguisher");
var version = require('../package.json').version;
var VALID_INCREMENTERS = ['simple', 'module', 'minimal'];
var EMPTY_CONFIG = "exports.default = {\n  incrementer: 'simple', // the incrementer to use ([minimal, simple, module])\n  types: ['cls', 'id'], // the types to rename (e.g. CSS classes, IDs)\n\n  inputDir: 'src/', // the input directory to use\n  outputDir: 'out/', // the output directory to use\n\n  exclude: [], // a regular expression array describing files to exclude from renaming\n};\n";
var DEFAULT_CONFIG_FN = 'distinguish.config.js';
var BINARY_NAME = 'distinguish';
var SPLASH_SCREEN = "      _ _     _   _                   _     _     \n     | (_)   | | (_)                 (_)   | |    \n   __| |_ ___| |_ _ _ __   __ _ _   _ _ ___| |__  \n  / _` | / __| __| | '_ \\ / _` | | | | / __| '_ \\ \n | (_| | \\__ \\ |_| | | | | (_| | |_| | \\__ \\ | | |\n  \\__,_|_|___/\\__|_|_| |_|\\__, |\\__,_|_|___/_| |_|\n                           __/ |                  \n                          |___/                   \n";
var CLIParser = /** @class */ (function () {
    function CLIParser(stream, parent) {
        this.stream = stream;
        this.parent = parent;
    }
    CLIParser.prototype.moreArguments = function () {
        if (this.stream.length == 0) {
            log_1.logStyle(log_1.FAIL, "Expected additional arguments\n");
            return false;
        }
        return true;
    };
    CLIParser.prototype.noMoreArguments = function () {
        if (this.stream.length != 0) {
            log_1.logStyle(log_1.FAIL, "Encountered extraneous arguments: " + this.stream + "\n");
            return false;
        }
        return true;
    };
    CLIParser.prototype.advance = function () {
        this.stream = this.stream.slice(1);
    };
    CLIParser.prototype.consume = function (s) {
        var token = this.stream[0];
        if (token == s)
            return true;
        return false;
    };
    CLIParser.prototype.consumeOption = function (forms) {
        var e_1, _a;
        try {
            for (var forms_1 = __values(forms), forms_1_1 = forms_1.next(); !forms_1_1.done; forms_1_1 = forms_1.next()) {
                var form = forms_1_1.value;
                if (this.consume(form))
                    return true;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (forms_1_1 && !forms_1_1.done && (_a = forms_1["return"])) _a.call(forms_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return false;
    };
    return CLIParser;
}());
var RenameCLI = /** @class */ (function (_super) {
    __extends(RenameCLI, _super);
    function RenameCLI() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RenameCLI.prototype.process = function () {
        var expectConfig = false;
        var expectIncrementer = false;
        var expectTypes = false;
        var expectInputDir = false;
        var expectOutputDir = false;
        var expectExclude = false;
        var noOpts = this.stream.length == 0;
        var opts = {};
        while (this.stream.length > 0) {
            // Go through expectations.
            if (expectIncrementer) {
                if (!this.moreArguments()) {
                    this.showUsage();
                    return;
                }
                var incrementer = this.stream[0];
                this.advance();
                if (VALID_INCREMENTERS.indexOf(incrementer) == -1) {
                    log_1.logStyle(log_1.FAIL, "Expected an incrementer in the set " + VALID_INCREMENTERS + "\n");
                    this.showUsage();
                    return;
                }
                opts.incrementer = incrementer;
                expectIncrementer = false;
                continue;
            }
            if (expectTypes) {
                if (!this.moreArguments()) {
                    this.showUsage();
                    return;
                }
                opts.types = this.stream[0].split(',');
                this.advance();
                expectTypes = false;
                continue;
            }
            if (expectInputDir) {
                if (!this.moreArguments()) {
                    this.showUsage();
                    return;
                }
                opts.inputDir = this.stream[0];
                this.advance();
                expectInputDir = false;
                continue;
            }
            if (expectOutputDir) {
                if (!this.moreArguments()) {
                    this.showUsage();
                    return;
                }
                opts.outputDir = this.stream[0];
                this.advance();
                expectOutputDir = false;
                continue;
            }
            if (expectExclude) {
                if (!this.moreArguments()) {
                    this.showUsage();
                    return;
                }
                opts.exclude = this.stream[0].split(',').map(function (x) { return new RegExp(x); });
                this.advance();
                expectExclude = false;
                continue;
            }
            // Consume options.
            if (this.consumeOption(['-c', '--config'])) {
                expectConfig = true;
                opts.configFile = DEFAULT_CONFIG_FN;
                this.advance();
                continue;
            }
            if (this.consumeOption(['-n', '--incrementer'])) {
                expectIncrementer = true;
                this.advance();
                continue;
            }
            if (this.consumeOption(['-t', '--types'])) {
                expectTypes = true;
                this.advance();
                continue;
            }
            if (this.consumeOption(['-i', '--inputDir'])) {
                expectInputDir = true;
                this.advance();
                continue;
            }
            if (this.consumeOption(['-o', '--outputDir'])) {
                expectOutputDir = true;
                this.advance();
                continue;
            }
            if (this.consumeOption(['-e', '--exclude'])) {
                expectExclude = true;
                this.advance();
                continue;
            }
            // If nothing was consumed and expectConfig is on, let's check for one.
            if (!this.moreArguments() || !expectConfig) {
                this.showUsage();
                return;
            }
            opts.configFile = this.stream[0];
            expectConfig = false;
            this.advance();
        }
        // Now that we're done processing, let's make sense of the args.
        // If no options were specified, put in the default config file.
        if (noOpts)
            opts.configFile = DEFAULT_CONFIG_FN;
        if (opts.configFile == null) {
            // Set defaults if no config specified.
            if (opts.incrementer == null)
                opts.incrementer = 'simple';
            if (opts.types == null)
                opts.types = ['cls', 'id'];
            if (opts.exclude == null)
                opts.exclude = [];
        }
        else {
            // Load in the config file.
            opts.configFile = path_1["default"].join(process.cwd(), opts.configFile);
            var settings = require(opts.configFile)["default"];
            if (opts.incrementer == null)
                opts.incrementer = settings.incrementer;
            if (opts.types == null)
                opts.types = settings.types;
            if (opts.inputDir == null)
                opts.inputDir = settings.inputDir;
            if (opts.outputDir == null)
                opts.outputDir = settings.outputDir;
            if (opts.exclude == null)
                opts.exclude = settings.exclude;
        }
        // Run time.
        console.log('Run settings:\n', opts);
        console.log();
        var distinguisher = new distinguisher_1.Distinguisher(opts);
        distinguisher.run();
    };
    RenameCLI.prototype.showUsage = function () {
        console.log("Usage: " + BINARY_NAME + " rename [options]\n\n  rename and namespace files recursively in a directory\n\nOptions:\n  -c, --config [fn]         Load all the settings from a config file.\n                            (default if no value passed: " + DEFAULT_CONFIG_FN + ")\n\nConfig options / overrides:\n  -n, --incrementer <str>   Specify the incrementer to use.\n                            Options are 'simple', 'module', or 'minimal'\n                            (default if no config specified: simple)\n\n  -t, --types <list>        Specify a list of types to rename\n                            (default if no config specified: cls,id)\n\n  -i, --inputDir <dir>      The input directory to use\n                            (this arg is mandatory if no config is specified)\n\n  -o, --outputDir <dir>     The output directory to use\n                            (this arg is mandatory if no config is specified)\n\n  -e, --exclude <list>      Regular expression of paths to exclude renaming.\n                            It is recommended to set this in a config file to\n                            have more control.\n                            (default: empty list)\n");
        process.exit(1);
    };
    return RenameCLI;
}(CLIParser));
var InitCLI = /** @class */ (function (_super) {
    __extends(InitCLI, _super);
    function InitCLI() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    InitCLI.prototype.process = function () {
        var fn = DEFAULT_CONFIG_FN;
        if (this.stream.length > 0) {
            fn = this.stream[0];
            this.advance();
            if (!this.noMoreArguments()) {
                this.showUsage();
                return;
            }
        }
        var startTime = Date.now();
        fs_1["default"].writeFileSync(fn, EMPTY_CONFIG);
        var deltaTime = Date.now() - startTime;
        log_1.logStyle(log_1.SUCCESS, "Wrote config to " + fn + " in " + deltaTime / 1000 + "s");
    };
    InitCLI.prototype.showUsage = function () {
        console.log("Usage: " + BINARY_NAME + " init <fn>\n\n  create a default distinguish config file\n\nArguments:\n  fn        The file to create.\n            (default: " + DEFAULT_CONFIG_FN + ")\n");
        process.exit(1);
    };
    return InitCLI;
}(CLIParser));
var CLI = /** @class */ (function (_super) {
    __extends(CLI, _super);
    function CLI() {
        var _this = _super.call(this, process.argv.slice(2)) || this;
        _this.version = version;
        return _this;
    }
    CLI.prototype.process = function () {
        // Try to consume options.
        if (this.consumeOption(['-v', '--version']))
            return this.showVersion();
        if (this.consumeOption(['-h', '--help', '--usage']))
            return this.showUsage();
        if (this.consumeOption(['--splash']))
            return this.showSplash();
        // Try to consume sub-commands.
        if (this.consumeOption(['rename'])) {
            return new RenameCLI(this.stream.slice(1), this).process();
        }
        if (this.consumeOption(['init'])) {
            return new InitCLI(this.stream.slice(1), this).process();
        }
        if (this.consumeOption(['help'])) {
            // Show help menus.
            this.advance();
            if (this.consumeOption(['rename'])) {
                return new RenameCLI(this.stream.slice(1), this).showUsage();
            }
            if (this.consumeOption(['init'])) {
                return new InitCLI(this.stream.slice(1), this).showUsage();
            }
        }
        this.showUsage();
    };
    CLI.prototype.showVersion = function () {
        console.log(this.version);
    };
    CLI.prototype.showUsage = function () {
        console.log("Usage: " + BINARY_NAME + " <command> [options]\n\nCommands:\n  rename [options]                   rename and namespace files\n                                     recursively in a directory\n\n  init <fn=" + DEFAULT_CONFIG_FN + ">    create a config file\n\n  help [command]                     get options for a given command\n\nOptions:\n  -v, --version                      print the version\n  -h, --help, --usage                print this message\n  --splash                           show a fun splash screen\n");
        process.exit(1);
    };
    CLI.prototype.showSplash = function () {
        console.log(SPLASH_SCREEN);
        log_1.logStyle(log_1.BOLD, 'Effortless renaming, minification, and namespacing');
        log_1.logStyle(log_1.BOLD, 'for CSS class names, IDs, and just about anything else.\n');
    };
    return CLI;
}(CLIParser));
new CLI().process();
