"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var renamer_1 = require("./renamer");
var namespec_1 = require("./namespec");
var incrementer_1 = require("./incrementer");
var log_1 = require("./log");
var NAMESPEC = '.namespec';
// Adapted from https://stackoverflow.com/a/34509653
function ensureDirectoryExistence(filePath) {
    var dirname = path_1["default"].dirname(filePath);
    if (fs_1["default"].existsSync(dirname))
        return;
    ensureDirectoryExistence(dirname);
    fs_1["default"].mkdirSync(dirname);
}
function getAllMatches(regex, str) {
    var result;
    var results = [];
    while ((result = regex.exec(str)) !== null) {
        results.push(result);
    }
    return results;
}
var incrementers = {
    minimal: incrementer_1.MinimalIncrementer,
    simple: incrementer_1.SimpleIncrementer,
    module: incrementer_1.ModuleIncrementer
};
var Distinguisher = /** @class */ (function () {
    function Distinguisher(distinguishConfig) {
        this.distinguishConfig = distinguishConfig;
        var incrementer = incrementers[distinguishConfig.incrementer];
        this.rootRenamer = new renamer_1.Renamer(incrementer, distinguishConfig.types);
    }
    Distinguisher.prototype.walkSync = function (dir, outDir, renamer, filelist) {
        var _this = this;
        if (dir === void 0) { dir = ''; }
        if (outDir === void 0) { outDir = ''; }
        if (filelist === void 0) { filelist = []; }
        var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e;
        if (!dir.endsWith('/'))
            dir += '/';
        if (!outDir.endsWith('/'))
            outDir += '/';
        var namespecPath = "" + dir + NAMESPEC;
        if (fs_1["default"].existsSync(namespecPath)) {
            // Parse a namespec file to determine the renamer's new scope.
            var namespec = new namespec_1.NamespecParser(fs_1["default"].readFileSync(namespecPath).toString()).parse();
            // Set the namespace.
            renamer = renamer.namespace(renamer_1.Renamer.pathSpecToParts(namespec.namespace));
            try {
                // Set imports.
                for (var _f = __values(namespec.imports.entries()), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var _h = __read(_g.value, 2), importName = _h[0], importMap = _h[1];
                    try {
                        for (var _j = __values(importMap.entries()), _k = _j.next(); !_k.done; _k = _j.next()) {
                            var _l = __read(_k.value, 2), type = _l[0], names = _l[1];
                            try {
                                for (var names_1 = __values(names), names_1_1 = names_1.next(); !names_1_1.done; names_1_1 = names_1.next()) {
                                    var name_1 = names_1_1.value;
                                    renamer["import"](importName, type, name_1);
                                }
                            }
                            catch (e_3_1) { e_3 = { error: e_3_1 }; }
                            finally {
                                try {
                                    if (names_1_1 && !names_1_1.done && (_c = names_1["return"])) _c.call(names_1);
                                }
                                finally { if (e_3) throw e_3.error; }
                            }
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_k && !_k.done && (_b = _j["return"])) _b.call(_j);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_g && !_g.done && (_a = _f["return"])) _a.call(_f);
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                // Set reserves.
                for (var _m = __values(namespec.reserves.entries()), _o = _m.next(); !_o.done; _o = _m.next()) {
                    var _p = __read(_o.value, 2), type = _p[0], reserves = _p[1];
                    try {
                        for (var _q = __values(reserves.values()), _r = _q.next(); !_r.done; _r = _q.next()) {
                            var reserveName = _r.value;
                            renamer.reserve(type, reserveName);
                        }
                    }
                    catch (e_5_1) { e_5 = { error: e_5_1 }; }
                    finally {
                        try {
                            if (_r && !_r.done && (_e = _q["return"])) _e.call(_q);
                        }
                        finally { if (e_5) throw e_5.error; }
                    }
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (_o && !_o.done && (_d = _m["return"])) _d.call(_m);
                }
                finally { if (e_4) throw e_4.error; }
            }
        }
        var files = fs_1["default"].readdirSync(dir == '' ? '.' : dir);
        files.forEach(function (file) {
            var e_6, _a;
            var fn = "" + dir + file;
            try {
                for (var _b = __values(_this.distinguishConfig.exclude), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var exclude = _c.value;
                    if (fn.match(exclude) != null)
                        continue;
                }
            }
            catch (e_6_1) { e_6 = { error: e_6_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                }
                finally { if (e_6) throw e_6.error; }
            }
            if (fs_1["default"].statSync(fn).isDirectory()) {
                filelist = _this.walkSync(fn + "/", "" + outDir + file, renamer, filelist);
            }
            else {
                filelist.push({
                    inputFile: "" + dir + file,
                    outputFile: "" + outDir + file,
                    renamer: renamer
                });
            }
        });
        return filelist;
    };
    Distinguisher.prototype.run = function () {
        var e_7, _a, e_8, _b, e_9, _c;
        var startTime = Date.now();
        try {
            for (var _d = __values(this.walkSync(this.distinguishConfig.inputDir, this.distinguishConfig.outputDir, this.rootRenamer)), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = _e.value, inputFile = _f.inputFile, outputFile = _f.outputFile, renamer = _f.renamer;
                var contents = fs_1["default"].readFileSync(inputFile).toString();
                var hadMatches = false;
                try {
                    for (var _g = __values(this.distinguishConfig.types), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var type = _h.value;
                        // Find all matches.
                        var matches = getAllMatches(new RegExp("_(" + type + ")-([a-zA-Z0-9_-]+)", 'g'), contents);
                        for (var i = matches.length - 1; i >= 0; i--) {
                            // Iterate in reverse order to safely overwrite.
                            hadMatches = true; // there was at least a match somewhere
                            var _j = __read(matches[i], 3), fullMatch = _j[0], typeMatch = _j[1], name_2 = _j[2];
                            var index = matches[i].index;
                            var renamed = renamer.addName(typeMatch, name_2);
                            contents =
                                contents.substr(0, index) +
                                    renamed +
                                    contents.substr(index + fullMatch.length);
                        }
                    }
                }
                catch (e_8_1) { e_8 = { error: e_8_1 }; }
                finally {
                    try {
                        if (_h && !_h.done && (_b = _g["return"])) _b.call(_g);
                    }
                    finally { if (e_8) throw e_8.error; }
                }
                if (hadMatches) {
                    log_1.logStyle(log_1.STATUS, "Writing " + outputFile + " with namespace " + renamer.namespaces.join('/'));
                }
                ensureDirectoryExistence(outputFile);
                fs_1["default"].writeFileSync(outputFile, contents.toString());
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
            }
            finally { if (e_7) throw e_7.error; }
        }
        var overallTime = Date.now() - startTime;
        log_1.logStyle(log_1.BOLD, "\nWrote output in " + overallTime / 1000 + "s");
        var danglers = this.rootRenamer.danglingImports();
        if (danglers.length > 0)
            console.log('\n');
        try {
            for (var danglers_1 = __values(danglers), danglers_1_1 = danglers_1.next(); !danglers_1_1.done; danglers_1_1 = danglers_1.next()) {
                var _k = danglers_1_1.value, sourceNamespace = _k.sourceNamespace, importNamespace = _k.importNamespace, type = _k.type, name_3 = _k.name;
                log_1.logStyle(log_1.WARN, "Dangling import: " + sourceNamespace + " imports unused {type: " + type + ", name: " + name_3 + "} from " + importNamespace);
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (danglers_1_1 && !danglers_1_1.done && (_c = danglers_1["return"])) _c.call(danglers_1);
            }
            finally { if (e_9) throw e_9.error; }
        }
    };
    return Distinguisher;
}());
exports.Distinguisher = Distinguisher;
