"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
var Incrementer = /** @class */ (function () {
    function Incrementer() {
        this.reserved = new Set();
    }
    Incrementer.prototype.next = function (name, renamer) {
        var renamed;
        do {
            renamed = this.next_(name, renamer);
        } while (this.reserved.has(renamed));
        return renamed;
    };
    Incrementer.prototype.reserve = function (name) {
        this.reserved.add(name);
    };
    return Incrementer;
}());
exports.Incrementer = Incrementer;
var SimpleIncrementer = /** @class */ (function (_super) {
    __extends(SimpleIncrementer, _super);
    function SimpleIncrementer() {
        var _this = _super.call(this) || this;
        _this.allNames = new Set();
        return _this;
    }
    SimpleIncrementer.prototype.next_ = function (name) {
        if (this.allNames.has(name)) {
            var baseName = name;
            // Start incrementing names.
            var i = 1;
            do {
                name = baseName + "_" + i;
            } while (this.allNames.has(name));
        }
        this.allNames.add(name);
        return name;
    };
    return SimpleIncrementer;
}(Incrementer));
exports.SimpleIncrementer = SimpleIncrementer;
function characterSafe(s) {
    return s.replace(/[^a-zA-Z0-9]/g, '');
}
var ModuleIncrementer = /** @class */ (function (_super) {
    __extends(ModuleIncrementer, _super);
    function ModuleIncrementer() {
        var _this = _super.call(this) || this;
        _this.allNames = new Set();
        return _this;
    }
    ModuleIncrementer.prototype.next_ = function (name, renamer) {
        name = "" + characterSafe(renamer.namespaces.slice(1).join('_')) + (renamer.namespaces.length > 1 ? '_' : '') + name;
        if (this.allNames.has(name)) {
            var baseName = name;
            // Start incrementing names.
            var i = 1;
            do {
                name = baseName + "_" + i;
            } while (this.allNames.has(name));
        }
        this.allNames.add(name);
        return name;
    };
    return ModuleIncrementer;
}(Incrementer));
exports.ModuleIncrementer = ModuleIncrementer;
var MinimalIncrementer = /** @class */ (function (_super) {
    __extends(MinimalIncrementer, _super);
    /**
     *
     * @param allowedCharacters A string containing all the allowed characters in variable names.
     * @param allowedFirstCharacters An optional string containing all the allowed characters at the start of variable names. If set, variable names will only start with characters in this string.
     */
    function MinimalIncrementer(allowedCharacters, allowedFirstCharacters) {
        if (allowedCharacters === void 0) { allowedCharacters = 'abcdefghijklmnopqrstuvwxyz0123456789'; }
        if (allowedFirstCharacters === void 0) { allowedFirstCharacters = 'abcdefghijklmnopqrstuvwxyz'; }
        var _this = _super.call(this) || this;
        _this.allowedCharacters = allowedCharacters;
        _this.length = 1; // current length of variable names being iterated.
        _this.index = 0; // an internal index to help iterate without storing names
        _this.allowedFirstCharacters =
            allowedFirstCharacters == null ? allowedCharacters : allowedFirstCharacters;
        return _this;
    }
    MinimalIncrementer.prototype.next_ = function () {
        // Use the current index and calculate all the positions in the allowed
        // character arrays by dividing and calculating the modulus at appropriate
        // strides.
        var i = this.index;
        // Store whether the current iteration is the last iteration at the current
        // length.
        var last = true;
        // Build the resulting string backwards by iterating and applying modulus
        // methods.
        var result = '';
        // Iterate through the allowed characters at any position in the string.
        for (var _ = 0; _ < this.length - 1; _++) {
            var index = i % this.allowedCharacters.length;
            if (index != this.allowedCharacters.length - 1)
                last = false;
            result = this.allowedCharacters.charAt(index) + result;
            // Integer divide i by the length of the allowed characters.
            i = (i / this.allowedCharacters.length) | 0;
        }
        // Finally, place the proper character from the allowed first characters at
        // the beginning of the resulting string.
        if (i != this.allowedFirstCharacters.length - 1)
            last = false;
        result = this.allowedFirstCharacters.charAt(i) + result;
        if (last) {
            // If the current iteration is the last one at the current length,
            // increment the length and reset the index.
            this.index = 0;
            this.length++;
        }
        else {
            // Otherwise, simply increment the index.
            this.index++;
        }
        return result;
    };
    return MinimalIncrementer;
}(Incrementer));
exports.MinimalIncrementer = MinimalIncrementer;
