"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
exports.__esModule = true;
var Renamer = /** @class */ (function () {
    function Renamer(incrementerType, types, namespaces, parent) {
        var e_1, _a;
        if (namespaces === void 0) { namespaces = ['root']; }
        this.incrementerType = incrementerType;
        this.types = types;
        this.namespaces = namespaces;
        this.parent = parent;
        this.childNamespaces = [];
        this.namingMaps = new Map();
        this.ownNamingMaps = new Map();
        this.childNamespaceMap = new Map();
        this.incrementers = new Map();
        this.imports = new Map();
        try {
            for (var types_1 = __values(types), types_1_1 = types_1.next(); !types_1_1.done; types_1_1 = types_1.next()) {
                var type = types_1_1.value;
                this.namingMaps.set(type, new Map());
                this.ownNamingMaps.set(type, new Map());
                this.incrementers.set(type, new incrementerType());
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (types_1_1 && !types_1_1.done && (_a = types_1["return"])) _a.call(types_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    Renamer.prototype.fullNamespace = function () {
        return this.namespaces.join('/');
    };
    /**
     * Creates a namespace specified by a single string, or returns the one if it already
     * exists.
     * @param name The namespace or its parts.
     */
    Renamer.prototype.namespace = function (name) {
        if (Array.isArray(name)) {
            if (name.length == 0)
                return this;
            return this.namespace(name[0]).namespace(name.slice(1));
        }
        if (name == '..') {
            var parent_1 = this.parent;
            if (parent_1 == null)
                throw new Error('Cannot backtrack from root');
            return parent_1;
        }
        if (name == '/') {
            return this.getRoot();
        }
        if (this.childNamespaceMap.has(name)) {
            return this.childNamespaceMap.get(name);
        }
        var renamer = new Renamer(this.incrementerType, this.types, this.namespaces.concat([name]), this);
        this.childNamespaces.push(renamer);
        this.childNamespaceMap.set(name, renamer);
        return renamer;
    };
    Renamer.pathSpecToParts = function (spec) {
        var parts = [];
        if (spec.startsWith('/')) {
            parts.push('/');
            spec = spec.substr(1);
        }
        if (spec.length > 0) {
            parts = parts.concat(spec.split('/'));
        }
        return parts;
    };
    Renamer.prototype.getRoot = function () {
        if (this.parent)
            return this.parent.getRoot();
        return this;
    };
    Renamer.prototype.increment = function (type, value, originalCaller) {
        if (originalCaller === void 0) { originalCaller = this; }
        if (this.parent != null)
            return this.parent.increment(type, value, originalCaller);
        var incrementer = this.incrementers.get(type);
        if (incrementer == null)
            throw new Error('Invalid type');
        return incrementer.next(value, originalCaller);
    };
    Renamer.prototype.addName = function (type, value, originalCaller) {
        if (originalCaller === void 0) { originalCaller = this; }
        var typeMap = this.namingMaps.get(type);
        var ownTypeMap = this.ownNamingMaps.get(type);
        var importMap = this.imports.get(type);
        var incrementer = this.incrementers.get(type);
        if (typeMap == null || incrementer == null)
            throw new Error('Invalid type');
        if (importMap != null) {
            if (importMap.has(value)) {
                var name_1 = importMap.get(value).addName(type, value, this);
                if (originalCaller == this) {
                    // Add to own type map if applicable.
                    ownTypeMap.set(value, name_1);
                }
                return name_1;
            }
        }
        if (typeMap.has(value)) {
            // Return early if the name is found.
            var name_2 = typeMap.get(value);
            if (originalCaller == this) {
                // Add to own type map if applicable.
                ownTypeMap.set(value, name_2);
            }
            return name_2;
        }
        var name = this.increment(type, value, originalCaller);
        typeMap.set(value, name);
        if (originalCaller == this) {
            // Add to own type map if applicable.
            ownTypeMap.set(value, name);
        }
        return name;
    };
    Renamer.prototype.setImport = function (type, name, renamer) {
        var importMap;
        if (this.imports.has(type)) {
            importMap = this.imports.get(type);
        }
        else {
            importMap = new Map();
            this.imports.set(type, importMap);
        }
        importMap.set(name, renamer);
    };
    Renamer.prototype["import"] = function (namespaceSpec, type, name) {
        var parts = Renamer.pathSpecToParts(namespaceSpec);
        var namespace = this.namespace(parts);
        this.setImport(type, name, namespace);
    };
    Renamer.prototype.reserve = function (type, name) {
        if (this.parent != null)
            this.parent.reserve(type, name);
        var incrementer = this.incrementers.get(type);
        if (incrementer == null)
            throw new Error("Cannot reserve: invalid type " + type);
        incrementer.reserve(name);
    };
    Renamer.prototype.danglingImports = function () {
        var e_2, _a, e_3, _b, e_4, _c;
        var danglers = [];
        try {
            for (var _d = __values(this.imports.entries()), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = __read(_e.value, 2), type = _f[0], importMap = _f[1];
                try {
                    // Go through each type-to-import entry.
                    for (var _g = __values(importMap.entries()), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var _j = __read(_h.value, 2), name_3 = _j[0], renamer = _j[1];
                        // Grab the import renamer instance, and use that to extract the typemap.
                        var typeMap = renamer.ownNamingMaps.get(type);
                        if (typeMap == null || !typeMap.has(name_3)) {
                            // The import is unused.
                            danglers.push({
                                sourceNamespace: this.fullNamespace(),
                                importNamespace: renamer.fullNamespace(),
                                type: type,
                                name: name_3
                            });
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_h && !_h.done && (_b = _g["return"])) _b.call(_g);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
            }
            finally { if (e_2) throw e_2.error; }
        }
        try {
            // Recurse
            for (var _k = __values(this.childNamespaces), _l = _k.next(); !_l.done; _l = _k.next()) {
                var child = _l.value;
                danglers = danglers.concat(child.danglingImports());
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_l && !_l.done && (_c = _k["return"])) _c.call(_k);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return danglers;
    };
    return Renamer;
}());
exports.Renamer = Renamer;
