"use strict";
exports.__esModule = true;
var renamer_1 = require("./renamer");
var tester_1 = require("./tester");
var namespec_1 = require("./namespec");
var incrementer_1 = require("./incrementer");
var t = new tester_1.Tester();
t.test('testBasicNaming', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['cls', 'id']);
    r.addName('cls', 'dog');
    r.addName('cls', 'cat');
    r.addName('cls', 'frog');
    var component = r.namespace('component1');
    component.addName('cls', 'machine');
    r.addName('cls', '1');
    component.addName('cls', 'dog');
    r.addName('cls', '2');
    r.addName('id', 'doggo');
    var rMap = r.namingMaps.get('cls');
    var rIdMap = r.namingMaps.get('id');
    var cMap = component.namingMaps.get('cls');
    t.assertEquals(rMap.get('dog'), 'a');
    t.assertEquals(rMap.get('cat'), 'b');
    t.assertEquals(rMap.get('frog'), 'c');
    t.assertEquals(cMap.get('machine'), 'd');
    t.assertEquals(rMap.get('1'), 'e');
    t.assertEquals(cMap.get('dog'), 'f');
    t.assertEquals(rMap.get('2'), 'g');
    t.assertEquals(rIdMap.get('doggo'), 'a');
});
t.test('deepNamespaces', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    var p3 = r.namespace(['p1', 'p2', 'p3']);
    t.assertEquals(p3.namespaces.join('/'), 'root/p1/p2/p3');
    var p2 = p3.parent;
    t.assert(p2);
    t.assertEquals(p2.namespaces.join('/'), 'root/p1/p2');
    var p1 = p2.parent;
    t.assert(p1);
    t.assertEquals(p1.namespaces.join('/'), 'root/p1');
    var p0 = p1.parent;
    t.assert(p0);
    t.assertEquals(p0, r);
    t.assertEquals(p0.namespaces.join('/'), 'root');
});
t.test('pathSpecToParts', function () {
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('/'), ['/']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog'), ['dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('/dog'), ['/', 'dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog/cat'), ['dog', 'cat']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('../dog'), ['..', 'dog']);
    t.assertArrayEquals(renamer_1.Renamer.pathSpecToParts('dog/cat/parrot'), [
        'dog',
        'cat',
        'parrot',
    ]);
});
t.test('namespaceSansImports', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    t.assertEquals(component.addName('id', 'dog'), 'b');
});
t.test('namespaceImportModuleToParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    component["import"]('..', 'id', 'dog');
    t.assertEquals(component.addName('id', 'dog'), 'a');
});
t.test('namespaceImportParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    t.assertEquals(component.addName('id', 'dog'), 'a');
});
t.test('namespecBasic', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('namespecReserves', function () {
    var spec = "\nnamespace slider\n\nreserve\n  css\n    cat\n\nfrom .. import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('namespecReservesAtEnd', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n\nreserve\n  css\n    cat\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
});
t.test('namespecMultipleValues', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    slider\n    oven\n    mitten\n  id\n    dog\n    cat\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css', 'id']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider', 'oven', 'mitten']);
    t.assertArrayEquals(Array.from(map.values())[1], ['dog', 'cat']);
});
t.test('namespecMultipleImports', function () {
    var spec = "\nnamespace slider\n\nfrom .. import\n  css\n    dog\n    cat\n    bark\n  id\n    yes\nfrom /slider import\n  css\n    slider\n";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(imports.keys()), ['..', '/slider']);
    var parentMap = imports.get('..');
    t.assertArrayEquals(Array.from(parentMap.keys()), ['css', 'id']);
    t.assertArrayEquals(Array.from(parentMap.values())[0], ['dog', 'cat', 'bark']);
    t.assertArrayEquals(Array.from(parentMap.values())[1], ['yes']);
    var sliderMap = imports.get('/slider');
    t.assertArrayEquals(Array.from(sliderMap.keys()), ['css']);
    t.assertArrayEquals(Array.from(sliderMap.values())[0], ['slider']);
});
t.test('namespecNoTrailingWhitespace', function () {
    var spec = "\nnamespace slider\n\nreserve\n  css\n    cat\n\nfrom .. import\n  css\n    slider";
    var parser = new namespec_1.NamespecParser(spec);
    var _a = parser.parse(), namespace = _a.namespace, imports = _a.imports, reserves = _a.reserves;
    t.assertEquals(namespace, 'slider');
    t.assertArrayEquals(Array.from(reserves.keys()), ['css']);
    t.assertArrayEquals(Array.from(Array.from(reserves.values())[0].values()), ['cat']);
    t.assertArrayEquals(Array.from(imports.keys()), ['..']);
    var map = imports.get('..');
    t.assertArrayEquals(Array.from(map.keys()), ['css']);
    t.assertArrayEquals(Array.from(map.values())[0], ['slider']);
});
t.test('simpleIncrementerBasic', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['cls', 'id']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    t.assertEquals(r.addName('id', 'dog'), 'dog');
    // Adding name again just reuses old name.
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
});
t.test('simpleIncrementerNamespace', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['cls']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    var module = r.namespace('module');
    t.assertEquals(module.addName('cls', 'dog'), 'dog_1');
});
t.test('moduleIncrementerNamespace', function () {
    var r = new renamer_1.Renamer(incrementer_1.ModuleIncrementer, ['cls']);
    t.assertEquals(r.addName('cls', 'dog'), 'dog');
    var module = r.namespace('module');
    t.assertEquals(module.addName('cls', 'dog'), 'module_dog');
    var badModule = r.namespace('mod^ule');
    t.assertEquals(badModule.addName('cls', 'dog'), 'module_dog_1');
});
t.test('importWarning', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['cls']);
    r["import"]('module', 'cls', 'dog');
    var danglers = r.danglingImports();
    t.assertEquals(danglers.length, 1);
    t.assertObjectEquals(danglers[0], {
        sourceNamespace: 'root',
        importNamespace: 'root/module',
        type: 'cls',
        name: 'dog'
    });
});
t.test('importOkModuleToParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    t.assertEquals(r.addName('id', 'dog'), 'a');
    var component = r.namespace('component');
    component["import"]('..', 'id', 'dog');
    // No dangling imports.
    t.assertEquals(r.danglingImports().length, 0);
});
t.test('importWarningParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    r.namespace('component');
    // Don't add id dog from component...
    // Some dangling imports.
    var danglers = r.danglingImports();
    t.assertEquals(danglers.length, 1);
    t.assertObjectEquals(danglers[0], {
        sourceNamespace: 'root',
        importNamespace: 'root/component',
        type: 'id',
        name: 'dog'
    });
});
t.test('importOkParentToModule', function () {
    var r = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    var component = r.namespace('component');
    component.addName('id', 'dog');
    // No dangling imports.
    t.assertEquals(r.danglingImports().length, 0);
});
t.test('danglingImportsRecursive', function () {
    var root = new renamer_1.Renamer(incrementer_1.MinimalIncrementer, ['id']);
    var r = root.namespace('root2');
    // Import before module even exists.
    r["import"]('component', 'id', 'dog');
    r.addName('id', 'dog');
    // const component = r.namespace('component');
    // Don't add id dog from component...
    // Some dangling imports.
    t.assertEquals(root.danglingImports().length, 1);
});
t.test('reservedIncrementer', function () {
    var m = new incrementer_1.MinimalIncrementer();
    m.reserve('b');
    t.assertEquals(m.next(), 'a');
    t.assertEquals(m.next(), 'c');
});
t.test('reservedRenamerChild', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['id']);
    var component = r.namespace('component');
    component.reserve('id', 'dog');
    var dog = r.addName('id', 'dog');
    t.assertEquals(dog, 'dog_1');
});
t.test('reservedRenamerParent', function () {
    var r = new renamer_1.Renamer(incrementer_1.SimpleIncrementer, ['id']);
    var component = r.namespace('component');
    r.reserve('id', 'dog');
    var dog = component.addName('id', 'dog');
    t.assertEquals(dog, 'dog_1');
});
